NAME

Log::Log4perl::Filter::MDC - Filter to match on values of a MDC key

SYNOPSIS

log4perl.filter.Match1               = Log::Log4perl::Filter::MDC
log4perl.filter.Match1.KeyToMatch    = foo
log4perl.filter.Match1.RegexToMatch  = bar

DESCRIPTION

This Log4perl filter checks if a predefined MDC key, as set in KeyToMatch, of the currently submitted message matches a predefined regex, as set in RegexToMatch.

SEE ALSO

Log::Log4perl::Filter, Log::Log4perl::Filter::Boolean, Log::Log4perl::Filter::LevelMatch, Log::Log4perl::Filter::LevelRange, Log::Log4perl::Filter::MDC, Log::Log4perl::Filter::StringMatch

LICENSE

Copyright 2002-2026 by Mike Schilli <m@perlmeister.com> and Kevin Goess <cpan@goess.org>.

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.